function updateBullets()
	moveBullets()
	hitEnemies()
	bulletsHitPlayer()
	despawnDistantBullets()
	bulletProxRocket()
	laserCollision()
end

function moveBullets()
	for i, bullet in ipairs(bullets) do
		bullet.px = bullet.x
		bullet.py = bullet.y
		bullet.x = bullet.x + math.cos(bullet.dir)*bullet.speed*hdt
		bullet.y = bullet.y + math.sin(bullet.dir)*bullet.speed*hdt
	end
end

function hitEnemies()
	for i, bullet in ipairs(bullets) do
		for j, enemy in ipairs(enemies) do
			
			local n = 25
			local angle = getAngle(bullet.px,bullet.py,bullet.x,bullet.y)
			local nx = (bullet.px-bullet.x)/n
			local ny = (bullet.py-bullet.y)/n
			while n > 0 do
				if getDistance(enemy.x,enemy.y,bullet.px+(nx*n),bullet.py+(ny*n)) < (10+10) then
					--the actual enemy hitting bit
					if bullet.friendly then
						enemy.hit = 0.1
						enemy.health = enemy.health - bullet.damage
						table.remove(bullets,i)
						--add fuel
						if enemy.health < 1 then
							enemy.killedByPlayer = true
							player.fuel = player.fuel + enemy.fuelGain
							player.totalFuel = player.totalFuel + enemy.scoreGain
							player.visualFuelTimer = 0.25
							--freeze
							freeze()
						end
						--play audio
						a_hit:setPitch(math.random(60,130)/100)
						a_hit:setVolume(1*Avolume)
						la.play(a_hit:clone())
						
						--add particles
						local n = 200*player.maxShootTimer
						while n > 0 do
							local rand = math.random(0,1)
							local angle = rand*180+math.rad(math.random(70,110))+bullet.dir
							newParticle = {id = math.random(1,10^21),x = enemy.x,y = enemy.y,dir = angle,velocity = math.random(0,160),pType = 2} table.insert(particles,newParticle)
							n = n - 1
						end
					end
					--end loop
					n = 0
				end
				n = n - 1
			end
				
		end
	end
end

function bulletProxRocket()
	--set proximity rocket
	for i, weapon in ipairs(weapons) do
		if i == player.weapon then
			player.weaponRadius = weapon.radius
			if weapon.isProxRocket then
				player.proxRocket = true
			else
				player.proxRocket = false
			end
		end
	end
	--update bullets
	for i, bullet in ipairs(bullets) do
		if bullet.isProxRocket then
			
			for j, enemy in ipairs(enemies) do
				if getDistance(enemy.x,enemy.y,bullet.x,bullet.y) < bullet.radius and bullet.friendly then
					table.remove(bullets,i)
					newExplosion = {x = bullet.x,y = bullet.y,radius = bullet.radius, friendly = true} table.insert(explosions,newExplosion)
					enemy.killedByPlayer = true
				end
			end
			--friendly
			if bullet.friendly then else
				if getDistance(player.x,player.y,bullet.x,bullet.y) < bullet.radius then
					table.remove(bullets,i)
					newExplosion = {x = bullet.x,y = bullet.y,radius = bullet.radius, friendly = true} table.insert(explosions,newExplosion)
				end
			end
			
		end
	end
end

function bulletsHitPlayer()
	for i, bullet in ipairs(bullets) do
		if getDistance(player.x,player.y,bullet.x,bullet.y) < (8+10) then
			if bullet.friendly then else
				player.health = player.health - 1
				table.remove(bullets,i)
			end
		end
	end
end

function despawnDistantBullets()
	for i, bullet in ipairs(bullets) do
		if getDistance(bullet.x,bullet.y,player.x,player.y) > 500 then
			table.remove(bullets,i)
		end
	end
end

function laserCollision()
	la.stop(a_screech)	--stopping the damn screeching
	for i, weapon in ipairs(weapons) do
		if i == player.weapon then
			if weapon.isLaser then
				for j, enemy in ipairs(enemies) do
					if getDistance(enemy.x,enemy.y,player.x,player.y) < 400 then
						if lk.isDown(controls.fire) then
						
							if checkIntersect({x=player.x,y=player.y}, {x=player.x+(math.cos(player.thrusterdir-math.rad(90))*400),y=player.y+(math.sin(player.thrusterdir-math.rad(90))*400)}, {x=enemy.x-10-5,y=enemy.y-10-5}, {x=enemy.x+10+5,y=enemy.y+10+5}) or
							checkIntersect({x=player.x,y=player.y}, {x=player.x+(math.cos(player.thrusterdir-math.rad(90))*400),y=player.y+(math.sin(player.thrusterdir-math.rad(90))*400)}, {x=enemy.x-10-5,y=enemy.y+10+5}, {x=enemy.x+10+5,y=enemy.y-10-5}) then
								enemy.hit = 0.1
								enemy.health = enemy.health - (weapon.damage*hdt)
								--add fuel
								if enemy.health < 1 then
									enemy.killedByPlayer = true
									player.fuel = player.fuel + enemy.fuelGain
									player.totalFuel = player.totalFuel + enemy.scoreGain
									player.visualFuelTimer = 0.25
									--freeze
									freeze()
								end
								--spawn particles
								local n = 2000*hdt
								local t = 3
								if enemy.r == pColor.r and enemy.g == pColor.g and enemy.b == pColor.b then
									t = 2
								end
								while n > 0 do
									newParticle = {id = math.random(1,10^21),x = enemy.x+math.random(-10,10),y = enemy.y+math.random(-10,10),dir = math.rad(math.random(0,360)),velocity = math.random(0,60),pType = t} table.insert(particles,newParticle)
									n = n - 1
								end
								--play sound
								a_screech:setPitch(2)
								a_screech:setVolume(1*Avolume)
								la.play(a_screech)
								
							end
							
						end
					end
				end
			end
			
			--laser thrust(Yes I am aware this doesn't work irl)
			if weapon.isLaser and lk.isDown(controls.fire) then
				newImpulse = {dir = player.thrusterdir+math.rad(180),intensity = player.weaponBackThrust,id = 0} table.insert(impulses,newImpulse)
			end
		end
	end
	
end