--require libs
require "libs/maid64"
local screen = require "libs/shack"
saveData = require("libs/saveData")

--require stuff
require "bin/player"
require "bin/impulses"
require "bin/bullets"
require "bin/enemies"
require "bin/blackholes"
require "bin/explosions"
require "bin/spawncontroller"
require "bin/particles"
require "bin/menu"
require "bin/functions"
require "bin/render/drawplayer"
require "bin/render/drawbullets"
require "bin/render/drawenemies"
require "bin/render/drawBG"
require "bin/render/drawblackholes"
require "bin/render/drawexplosions"
require "bin/render/drawparticles"
require "bin/render/drawmenu"
require "bin/render/drawsettings"
require "bin/render/drawhud"

--variables
lg = love.graphics
lk = love.keyboard
lm = love.mouse
la = love.audio
lf = love.filesystem
globalTimer = 0
gameTimer = 0
currentFrame = 0
shake = 0
shakeFade = 0
shakeFadeAmmount = 25
desiredEnemyCount = 110
thrustParticleTimer = 0
gamestate = 'menu'
Avolume = 0.3
fuelPitch = 1
fuelPitchTimer = 0
rand2 = 1
menuStarAngle = 20

--set filter
lg.setDefaultFilter("nearest", "nearest",0)

--get imgs
i_player1 = lg.newImage("assets/player/player.png")
i_player2 = lg.newImage("assets/player/player2.png")
i_player3 = lg.newImage("assets/player/player3.png")
i_player4 = lg.newImage("assets/player/player4.png")
i_player5 = lg.newImage("assets/player/player5.png")
i_player6 = lg.newImage("assets/player/player6.png")
i_player7 = lg.newImage("assets/player/player7.png")
i_player8 = lg.newImage("assets/player/player8.png")
i_player9 = lg.newImage("assets/player/player9.png")
i_player10 = lg.newImage("assets/player/player10.png")
i_player11 = lg.newImage("assets/player/player11.png")
i_playerSimple = lg.newImage("assets/player/playersimple.png")
i_playerSecret = lg.newImage("assets/player/secretdick.png")
i_player = i_player1
i_asteroid = lg.newImage("assets/enemies/asteroid.png")
i_asteroid2 = lg.newImage("assets/enemies/asteroid2.png")
i_asteroid3 = lg.newImage("assets/enemies/asteroid3.png")
i_asteroid4 = lg.newImage("assets/enemies/asteroid4.png")
i_asteroid5 = lg.newImage("assets/enemies/asteroid5.png")
i_fighter = lg.newImage("assets/enemies/fighter.png")
i_hunter = lg.newImage("assets/enemies/hunter.png")
i_cannon = lg.newImage("assets/enemies/cannon.png")
i_mine = lg.newImage("assets/enemies/mine.png")
i_blur = lg.newImage("assets/effects/blur.png")
i_chevron = lg.newImage("assets/effects/arrow.png")
i_blackhole = lg.newImage("assets/effects/blackhole.png")
i_blackholeVortex = lg.newImage("assets/effects/blackholevortex.png")
i_debris = lg.newImage("assets/effects/debris.png")
i_flames = lg.newImage("assets/effects/flames.png")
i_fuelParticle = lg.newImage("assets/effects/fuel.png")
i_bullet1 = lg.newImage("assets/bullets/bullet1.png")
i_rocket = lg.newImage("assets/bullets/rocket.png")
i_heart = lg.newImage("assets/hud/heart.png")
i_cross = lg.newImage("assets/hud/cross.png")
i_circle = lg.newImage("assets/hud/circle.png")
i_fuel = lg.newImage("assets/hud/fuel.png")
i_logo = lg.newImage("assets/menu/logo.png")
i_cursor = lg.newImage("assets/menu/cursor.png")
i_settings = lg.newImage("assets/menu/settingsicon.png")
i_play = lg.newImage("assets/menu/playicon.png")
i_return = lg.newImage("assets/menu/returnicon.png")
i_fullscreen = lg.newImage("assets/menu/fullscreenicon.png")
i_fullscreenOff = lg.newImage("assets/menu/fullscreenofficon.png")
i_select = lg.newImage("assets/menu/selecticon.png")
i_turnLeft = lg.newImage("assets/menu/turnleft.png")
i_turnRight = lg.newImage("assets/menu/turnright.png")
i_accelerate = lg.newImage("assets/menu/accelerate.png")
i_fire = lg.newImage("assets/menu/fire.png")
i_quit = lg.newImage("assets/menu/quiticon.png")
i_questionmark = lg.newImage("assets/menu/questionmark.png")

--sounds
a_explosion = la.newSource("assets/audio/explosion.wav")
a_fire1 = la.newSource("assets/audio/fire1.wav")
a_laser = la.newSource("assets/audio/laser.wav")
a_screech = la.newSource("assets/audio/screech.wav")
a_hit = la.newSource("assets/audio/hit.wav")
a_health = la.newSource("assets/audio/health.wav")
a_blip = la.newSource("assets/audio/blip.wav")
a_super = la.newSource("assets/audio/super.wav")

--canvases
c_playerMuzzleFlash = lg.newCanvas(400,400)
c_playerChargeFlash = lg.newCanvas(400,400)
c_playerThrust = lg.newCanvas(400,400)
c_hudShadow = lg.newCanvas(640,360)



function love.load(arg)
	--set up maid64
	maid64.setup(1280/2,720/2)
	--set up scores
	if lf.exists("highscores") then
		scores = saveData.load("highscores")
	else
		scores = {}
		saveData.save(scores,"highscores")
	end
	--set up controls
	if lf.exists("controls") then
		controls = saveData.load("controls")
	else
		controls = {turnLeft = lk.getScancodeFromKey('a'),turnRight = lk.getScancodeFromKey('d'),accelerate = lk.getScancodeFromKey('w'),fire = lk.getScancodeFromKey('space'),respawn = lk.getScancodeFromKey('r')}
		saveData.save(controls,"controls")
	end
	
	
	--tables
	player = {x = 640/2,y = 360/2,health = 5,maxHealth = 5,dir = 0,velocity = 0,thrusterdir = 0,turnSpeed = math.rad(180),thrusterPower = 175,ammount2 = 1,maxShootTimer = 0.1,shootTimer = 0,shootTimer2 = 0,weapon = 1,pWeapon = -1,weaponBackThrust = 175,oobTimer = 1,fuel = 1,hitTimer = 0,visualFuelTimer  = 0, healthTimer = 0, weaponRadius = 0, totalFuel = 0, muzzleTimer = 0, frenzyTimer = 0}
	PA = {x = -4000,y = -4000,w = 8000,h = 8000}
	settings = {fullscreen = false,simplePlayer = false,secretPlayer = false}
	pColor = {r = 255,g = 255,b = 255}	--neutral
	sColor = {r = 255,g = 0,b = 0}	--bad
	tColor = {r = 0,g = 255,b = 0}	--good
	qColor = {r = 0,g = 0,b = 0}	--dark stuff
	t2Color = {r = 0,g = 0,b = 0}	--temporary color space
	impulses = {}
	bullets = {}
	enemies = {}
	stars = {}
	blackholes = {}
	explosions = {}
	weapons = {}
	particles = {}
	buttons = {}
	
	--load functions. Stuff like getDistance() and the like
	loadFunctions()
	
	--add weapons
	--(x1,y1,dir1,friendly1,damage1,ammount,accuracy,velocity)
	newWeapon = {damage = 9,ammount = 1,accuracy = 5,velocity = 650*1.3,firerate = 0.09, firerate2 = 0, ammount2 = 1, thrust = 250, isLaser = false, isCharge = false, radius = nil} table.insert(weapons,newWeapon)	--machinegun
	newWeapon = {damage = 1.75,ammount = 3,accuracy = 10,velocity = 800*1.3,firerate = 0.07, firerate2 = 0, ammount2 = 1, thrust = 230, isLaser = false, isCharge = false, radius = nil} table.insert(weapons,newWeapon)	--typhoon
	newWeapon = {damage = 3,ammount = 20,accuracy = 15,velocity = 350*1.3,firerate = 0.6, firerate2 = 0, ammount2 = 1, thrust = 1450, isLaser = false, isCharge = false, radius = nil} table.insert(weapons,newWeapon)	--shotgun
	newWeapon = {damage = 75,ammount = 1,accuracy = 0,velocity = 1300*1.3,firerate = 0.375, firerate2 = 0, ammount2 = 1, thrust = 1250, isLaser = false, isCharge = false, radius = nil} table.insert(weapons,newWeapon)	--ranged thing
	newWeapon = {damage = 50,ammount = 1,accuracy = 15,velocity = 410*1.3,firerate = 0.75, firerate2 = 0.03, ammount2 = 7, thrust = 700, isLaser = false, isCharge = true, isProxRocket = true, radius = 25} table.insert(weapons,newWeapon)	--charge burst rocket swarm
	newWeapon = {damage = 50,ammount = 1,accuracy = 0,velocity = 250*1.3,firerate = 0.5, firerate2 = 0, ammount2 = 1, thrust = 1600, isLaser = false, isCharge = false, isProxRocket = true, radius = 40} table.insert(weapons,newWeapon)	--proximity rocket
	newWeapon = {damage = 5,ammount = 4,accuracy = 15,velocity = 240*1.3,firerate = 0.75, firerate2 = 0, ammount2 = 1, thrust = 1800, isLaser = false, isCharge = false, isProxRocket = true, radius = 30} table.insert(weapons,newWeapon)	--proximity rocket swarm
	newWeapon = {damage = 50,ammount = 0,accuracy = 15,velocity = 260*1.3,firerate = 0.1, firerate2 = 0, ammount2 = 1, thrust = 65, isLaser = true, isCharge = false, radius = nil} table.insert(weapons,newWeapon)	--laser
	newWeapon = {damage = 20,ammount = 1,accuracy = 0,velocity = 1500*1.3,firerate = 0.75, firerate2 = 0.03, ammount2 = 5, thrust = 230, isLaser = false, isCharge = true, isProxRocket = false, radius = nil} table.insert(weapons,newWeapon)	--charged shot
	newWeapon = {damage = 20,ammount = 1,accuracy = 3,velocity = 800*1.3,firerate = 0.6, firerate2 = 0.04, ammount2 = 3, thrust = 800, isLaser = false, isCharge = false, radius = nil} table.insert(weapons,newWeapon)	--burst
	newWeapon = {damage = 50,ammount = 1,accuracy = 0,velocity = 500*1.3,firerate = 0.2, firerate2 = 0, ammount2 = 1, thrust = 0, isLaser = false, isCharge = false, isProxRocket = true, radius = 40} table.insert(weapons,newWeapon)	--SUPER
	
	--add menu buttons
	newButton = {x = 610, y = 330, img = i_quit, scale = 3, state = 'menu', id = 0} table.insert(buttons,newButton)	--quit
	newButton = {x = 200, y = 200, img = i_play, scale = 3, state = 'menu', id = 1} table.insert(buttons,newButton)	--play
	newButton = {x = 400, y = 200, img = i_settings, scale = 3, state = 'menu', id = 2} table.insert(buttons,newButton)	--goto settings
	newButton = {x = 30, y = 360-30, img = i_return, scale = 3, state = 'settings', id = 3} table.insert(buttons,newButton)	--return to mainmenu from settings
	newButton = {x = 40, y = 40+50, img = i_fullscreen, scale = 3.1, state = 'settings', id = 4} table.insert(buttons,newButton)	--fullscreen on
	newButton = {x = 110, y = 40+50, img = i_fullscreenOff, scale = 3.1, state = 'settings', id = 5} table.insert(buttons,newButton)	--fullscreen off
	newButton = {x = 40, y = 110+100, img = i_player1, scale = 3.1, state = 'settings', id = 6} table.insert(buttons,newButton)	--simple player on
	newButton = {x = 110, y = 110+100, img = i_playerSimple, scale = 3.1, state = 'settings', id = 7} table.insert(buttons,newButton)	--simple player off
	--controls
	newButton = {x = 450, y =40+50, img = i_turnLeft, scale = 3.1, state = 'settings', id = 8} table.insert(buttons,newButton)	--set turnLeft controls
	newButton = {x = 450, y =40+120, img = i_turnRight, scale = 3.1, state = 'settings', id = 9} table.insert(buttons,newButton)	--set turnRight controls
	newButton = {x = 450, y =40+190, img = i_accelerate, scale = 3.1, state = 'settings', id = 10} table.insert(buttons,newButton)	--set accelerate controls
	newButton = {x = 450, y =40+260, img = i_fire, scale = 3.1, state = 'settings', id = 11} table.insert(buttons,newButton)	--set fire controls
	--secret player character(shhh don't tell anyone)
	for i, score in ipairs(scores) do
		if score.value > 50000 then
			newButton = {x = 180, y = 110+100, img = i_questionmark, scale = 3.1, state = 'settings', id = 69} table.insert(buttons,newButton)	--secret
		end
	end
	
end

function love.update(dt)
	hdt = dt
	globalTimer = globalTimer + hdt
	currentFrame = currentFrame + 1
	
	--set player pHealth
	player.pHealth = player.health
	
	--remove cursor
	lm.setVisible(false)
	
	--set mouse stuff
	mouse = {x = lm.getX(),y = lm.getY()}
	
	--update stuff
	if gamestate == 'playing' then
		updateBullets()
		updateSpawnController()
		updateEnemies()
		updateBlackholes()
		updatePlayer()
		updateExplosions()
		updateParticles()
		updateImpulses()
		gameTimer = gameTimer + hdt
	else
		updateMenu()
	end
	
	
end

function love.draw(dt)
	--apply screenshake
	screen:apply()

	maid64.start()--starts the maid64 process
	
	if gamestate == 'playing' then	--draw stuff in the game part of this game
		drawBG()
		drawBlackholes()
		drawPlayer()
		drawEnemies()
		drawBullets()
		drawExplosions()
		drawParticles()
		drawHUD()
	elseif gamestate == 'menu' then	--draw mainmenu
		drawBG()
		drawMenu()
	elseif gamestate == 'settings' then	--draw the settings menu
		drawBG()
		drawSettingsMenu()
	end
	
	--set screenshake
	screen:setShake(shake+shakeFade)
	shakeFade = shakeFade - (shakeFadeAmmount*hdt)
	if shakeFade < 0 then shakeFade = 0 end
	
	lg.setColor(255,255,255)
	maid64.finish()--finishes the maid64 process
end

function love.resize(w, h)
    -- this is used to resize the screen correctly
    maid64.resize(w, h)
end
